
/**************************************************************************
**  (c) Copyright 2002, Andromeda Technology & Automation
** This is free software; you can redistribute it and/or modify it under the
** terms of the GNU General Public License, see the file COPYING.
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : database.cpp
**      SYSTEM NAME    : Gnucomo - Gnu Computer Monitoring
**      VERSION NUMBER : $Revision: 1.6 $
**
**  DESCRIPTION      :  Implementation of the gnucomo database classes
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Sep 10, 2002
**      LAST UPDATE     : Jan 31, 2003
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: database.cpp,v $
   Revision 1.6  2003/02/05 09:33:42  arjen
   gnucomo_database::new_notification() retruns the id number of the
   newly created notification record.

   Revision 1.5  2003/01/20 07:31:42  arjen
   Removed some debug output.

   Revision 1.4  2003/01/18 08:52:32  arjen
   New C++ function: gnucomo_database::new_notification()

   Revision 1.3  2002/11/09 08:04:27  arjen
   Added a reference to the GPL

   Revision 1.2  2002/11/04 10:13:36  arjen
   Use proper namespace for iostream classes

   Revision 1.1  2002/10/05 10:25:49  arjen
   Creation of gcm_input and a first approach to a web interface

*****************************/

static const char *RCSID = "$Id: database.cpp,v 1.6 2003/02/05 09:33:42 arjen Exp $";

#include <AXE/date.h>

#include "database.h"

extern bool verbose;   /*  Defined in the main application */

/*=========================================================================
**  NAME           : gnucomo_database
**  SYNOPSIS       : gnucomo_database(gnucomo_config &c);
**  PARAMETERS     : 
**  RETURN VALUE   : Database constructor. Establishes a connection with
**                   the database server.
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Sep 26, 2002
**=========================================================================
*/

gnucomo_database::gnucomo_database(gnucomo_config *c)
{
   cfg = c;

   if (verbose)
   {
      std::cout <<  "Database connection string = " << cfg->Database() << "\n";
   }

   db = new PgDatabase(cfg->Database());

   if (db->ConnectionBad())
   {
      std::cerr << "Can not connect to database: " << db->ErrorMessage();
   }
}

/*=========================================================================
**  NAME           : find_host
**  SYNOPSIS       : String gnucomo_database::find_host(String hostname);
**  PARAMETERS     : 
**  RETURN VALUE   : Find a hostname in the 'object' table of the gnucomo database
**                   and return its object id.
**                   Return an empty string as objectid if the hostname is
**                   not found.
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Sep 16, 2002
**=========================================================================
*/

String gnucomo_database::find_host(const String hostname)
{
   String objectid("");
   String check_host("select objectid from object where ");

   check_host += "objectname = '";
   check_host += hostname;
   check_host += "'";

   if (Query(check_host) > 0)
   {
      objectid = String(db->GetValue(0, "objectid"));
   }

   return objectid;
}

/*
 *  Create a new notification with an action_user and return the notification id
 */

String gnucomo_database::new_notification(String objectid, String issue, String remark)
{
   String qry;
   UTC    now = Now();

   String insertion;
   String notif_id("");

   String issueid("");

   std::cout << "Creating notification for " << issue << ": " << remark << "\n";

   qry = "select type_of_issueid, suggested_priority from type_of_issue where name='";
   qry += issue + "'";
   if (Query(qry) == 1)
   {
      issueid = Field(0, "type_of_issueid");
      insertion = "insert into notification (objectid, type_of_issueid, timestamp, ";
      insertion += "   statuscode, priority) values ('";
      insertion += objectid + "', '";
      insertion += issueid + "', '" + now.format("%Y-%m-%d %T") + "', 'new', '";
      insertion += Field(0, "suggested_priority") + "')";

      qry = "select notificationid from notification where objectid='";
      qry += objectid + "' and type_of_issueid = '";
      qry += issueid + "' order by notificationid";

      Query(insertion);
      int tuples = Query(qry);

      if (tuples > 0)
      {
         notif_id = Field(tuples - 1, "notificationid");

         insertion = "insert into action_user (actionid, username, notificationid,";
         insertion += "    timestamp, statuscode, remarks) values ('1', 'gnucomo', '";
         insertion += notif_id + "', '" + now.format("%Y-%m-%d %T") + "', 'new', '";
         insertion += remark + "')";

         Query(insertion);
      }
      else
      {
         std::cerr << "Error inserting notification.\n";
      }
   }

   return notif_id;
}

